@@ -13,6 +13,9 @@ from pytype.typegraph import cfg_utils
 
 log = logging.getLogger(__name__)
 
+# Type aliases
+ArgsDict = Dict[str, cfg.Variable]
+
 # We can't import abstract here due to a circular dep.
 _BaseValue = Any  # abstract.BaseValue
 _TypeParameter = Any  # abstract.TypeParameter
@@ -767,6 +770,18 @@ def maybe_unwrap_decorated_function(func):
   return func.func
 
 
+def unwrap_final(val):
+  """Unwrap Final[T] -> T."""
+  if _isinstance(val, "FinalAnnotation"):
+    # Final type created via an annotation in the current module
+    return val.annotation
+  elif (_isinstance(val, "Instance") and val.cls.full_name == "typing.Final"):
+    # Final types loaded from a pyi file get converted to abstract.Instance
+    # with cls=typing.Final and instance type parameter T
+    return get_atomic_value(val.get_instance_type_parameter(T))
+  return val
+
+
 # The _isinstance and _make methods should be used only in pytype.abstract
 # submodules that are unable to reference abstract.py classes due to circular
 # dependencies. To prevent accidental misuse, the methods are marked private.
@@ -817,3 +832,12 @@ def is_recursive_annotation(annot):
 def is_ellipsis(val):
   return (val == val.ctx.convert.ellipsis or
           (is_concrete(val) and val.pyval == "..."))
+
+
+def update_args_dict(args: ArgsDict, update: ArgsDict, node: cfg.CFGNode):
+  """Update a {str: Variable} dict by merging bindings."""
+  for k, v in update.items():
+    if k in args:
+      args[k].PasteVariable(v, node)
+    else:
+      args[k] = v
