@@ -158,7 +158,7 @@ FEATURE_FLAGS = [
       "'typing.Tuple' compatibility.")),
     ("--strict-parameter-checks", False,
      "Enable exhaustive checking of function parameter types."),
-    ("--enable-nested-classes", False,
+    ("--enable-nested-classes", True,
      "Enable support for nested classes in .py files."),
     ("--strict-primitive-comparisons", False,
      "Emit errors for comparisons between incompatible primitive types."),
@@ -172,6 +172,10 @@ FEATURE_FLAGS = [
      "Enable parameter type checks for overriding methods."),
     ("--overriding-return-type-checks", False,
      "Enable return type checks for overriding methods."),
+    ("--enable-bare-annotations", False,
+     "Support variable annotations without assignments."),
+    ("--enable-cached-property", False,
+     "Support pyglib's @cached.property."),
 ]
 
 
