@@ -65,6 +65,8 @@ class Module:
     has_unresolved_pointers: Whether all ClassType pointers have been filled in
   """
 
+  _INIT_NAMES = ("__init__.pyi", f"__init__.{pytd_utils.PICKLE_EXT}")
+
   # pylint: disable=redefined-outer-name
   def __init__(self, module_name, filename, ast,
                pickle=None, has_unresolved_pointers=True):
@@ -83,7 +85,7 @@ class Module:
       # imports_map_loader adds os.devnull entries for __init__.py files in
       # intermediate directories.
       return True
-    return self.filename and os.path.basename(self.filename) == "__init__.pyi"
+    return self.filename and os.path.basename(self.filename) in self._INIT_NAMES
 
 
 class BadDependencyError(Exception):
@@ -200,6 +202,13 @@ class _ModuleMap:
         todo.append(self._modules[module_prefix])
       newly_loaded_asts.append(loaded_ast)
       m.ast = loaded_ast.ast
+      if loaded_ast.is_package:
+        init_file = f"__init__.{pytd_utils.PICKLE_EXT}"
+        if m.filename and os.path.basename(m.filename) != init_file:
+          base, _ = os.path.splitext(m.filename)
+          m.filename = os.path.join(base, init_file)
+        else:
+          m.filename = self.PREFIX + os.path.join(m.module_name, init_file)
       m.pickle = None
     module_map = self.get_module_map()
     for loaded_ast in newly_loaded_asts:
@@ -390,10 +399,13 @@ class Loader:
     # We assume that the Loader is in a consistent state here. In particular, we
     # assume that for every module in _modules, all the transitive dependencies
     # have been loaded.
+    # pylint: disable=g-complex-comprehension
     items = tuple(
         (name, serialize_ast.StoreAst(
-            module.ast, open_function=self.options.open_function))
+            module.ast, open_function=self.options.open_function,
+            is_package=module.is_package()))
         for name, module in sorted(self._modules.items()))
+    # pylint: enable=g-complex-comprehension
     # Preparing an ast for pickling clears its class pointers, making it
     # unsuitable for reuse, so we have to discard the builtins cache.
     builtin_stubs.InvalidateCache()
@@ -408,7 +420,7 @@ class Loader:
       lookup_ast = lookup_ast or mod_ast
       self._load_ast_dependencies(dependencies, lookup_ast)
       mod_ast = self._resolve_external_types(
-          mod_ast, lookup_ast and lookup_ast.name)
+          mod_ast, lookup_ast=lookup_ast)
     mod_ast = self._resolver.resolve_local_types(mod_ast, lookup_ast=lookup_ast)
     return mod_ast
 
@@ -508,7 +520,13 @@ class Loader:
             continue
           else:
             self._path_finder.log_module_not_found(name)
-            raise BadDependencyError("Can't find pyi for %r" % name, ast_name)
+            try:
+              pytd.LookupItemRecursive(lookup_ast, name)
+            except KeyError as e:
+              raise BadDependencyError(
+                  "Can't find pyi for %r" % name, ast_name) from e
+            # This is a dotted local reference, not an external reference.
+            continue
       # If `name` is a package, try to load any base names not defined in
       # __init__ as submodules.
       if not self._modules[name].is_package() or "__getattr__" in dep_ast:
@@ -529,8 +547,11 @@ class Loader:
             # result - resolve_external_types will raise a better error.
             self._path_finder.log_module_not_found(full_name)
 
-  def _resolve_external_types(self, mod_ast, mod_name=None):
+  def _resolve_external_types(self, mod_ast, lookup_ast=None):
     module_map = self._modules.get_module_map()
+    mod_name = lookup_ast and lookup_ast.name
+    if mod_name and mod_name not in module_map:
+      module_map[mod_name] = lookup_ast
     mod_ast = self._resolver.resolve_external_types(
         mod_ast, module_map, self._aliases, mod_name=mod_name)
     return mod_ast
@@ -612,13 +633,17 @@ class Loader:
     if mod_ast:
       try:
         self._resolver.verify(mod_ast)
-      except (BadDependencyError, visitors.ContainerError):
+      except (BadDependencyError, visitors.ContainerError) as e:
         # In the case of a circular import, an external type may be left
         # unresolved, so we re-resolve lookups in this module and its direct
         # dependencies. Technically speaking, we should re-resolve all
         # transitive imports, but lookups are expensive.
         dependencies = self._resolver.collect_dependencies(mod_ast)
         for k in dependencies:
+          if k not in self._modules:
+            raise (
+                BadDependencyError("Can't find pyi for %r" % k, mod_ast.name)
+            ) from e
           self._modules[k].ast = self._resolve_external_types(
               self._modules[k].ast)
         mod_ast = self._resolve_external_types(mod_ast)
