@@ -362,7 +362,7 @@ class LookupExternalTypes(_RemoveTypeParametersFromGenericAny, _ToTypeVisitor):
     self._alias_name = None
     self._in_generic_type = 0
     self._star_imports = set()
-    self._unit = None
+    self._unit = module_map.get(self_name)
 
   def _ResolveUsingGetattr(self, module_name, module):
     """Try to resolve an identifier using the top level __getattr__ function."""
@@ -514,10 +514,21 @@ class LookupExternalTypes(_RemoveTypeParametersFromGenericAny, _ToTypeVisitor):
     getattrs = set()
     ast = self._module_map[module]
     type_param_names = set()
+    if module == "http.client":
+      # NOTE: http.client adds symbols to globals() at runtime, which is not
+      # reflected in its typeshed pyi file. The simplest fix is to ignore
+      # __all__ for star-imports of that file.
+      exports = None
+    else:
+      exports = [x for x in ast.constants if x.name.endswith(".__all__")]
+      if exports:
+        exports = exports[0].value
     for member in sum((ast.constants, ast.type_params, ast.classes,
                        ast.functions, ast.aliases), ()):
       _, _, member_name = member.name.rpartition(".")
-      if member_name == "__all__":
+      if exports and member_name not in exports:
+        # Not considering the edge case `__all__ = []` since that makes no
+        # sense in practice.
         continue
       new_name = self._ModulePrefix() + member_name
       if isinstance(member, pytd.Function) and member_name == "__getattr__":
@@ -1197,6 +1208,10 @@ class AddNamePrefix(Visitor):
       # This is an external type; do not prefix it. StripExternalNamePrefix will
       # remove it later.
       return node
+    if node.name.split(".")[0] in self.classes:
+      # We need to check just the first part, in case we have a class constant
+      # like Foo.BAR, or some similarly nested name.
+      return node.Replace(name=self.prefix + node.name)
     if self.cls_stack:
       if node.name == self.cls_stack[-1].name:
         # We're referencing a class from within itself.
@@ -1208,10 +1223,6 @@ class AddNamePrefix(Visitor):
           # ImmediateOuter.Nested, so we need to insert the full class stack.
           name = self.prefix + self._ClassStackString() + "." + base
           return node.Replace(name=name)
-    if node.name.split(".")[0] in self.classes:
-      # We need to check just the first part, in case we have a class constant
-      # like Foo.BAR, or some similarly nested name.
-      return node.Replace(name=self.prefix + node.name)
     return node
 
   def VisitClass(self, node):
