@@ -1,15 +1,10 @@
-#!/usr/bin/env python
-
 """Generate cross references from a project."""
 
-from __future__ import print_function
-
 import signal
 import sys
 import traceback
 
 from pytype import utils
-from pytype.pytd.parse import node
 
 from pytype.tools.xref import debug
 from pytype.tools.xref import indexer
@@ -25,8 +20,6 @@ def main():
     print(str(e), file=sys.stderr)
     sys.exit(1)
 
-  node.SetCheckPreconditions(options.check_preconditions)
-
   if options.timeout is not None:
     signal.alarm(options.timeout)
 
